$NOLIST
$NOSB
$NOMOD51
$INCLUDE (REG152.ASM)
$LIST

;*********************************************************************
;THIS PROGRAM IS WRITTEN TO START A TRANSMISSION IN CSMA/CD MODE UNDER 
;CPU CONTROL AND ASSUMES SOME TRANSMISSIONS WILL BE IN ERROR, THUS THE 
;NAME Transmit Error Under Cpu Control (TEUCC2).  THIS PROGRAM WAS BUILT 
;ON THE PROGRAM, TVUCC2, AND FOLLOWS THE SAME FLOW FOR A VALID TRANSMITTED 
;FRAME.
;
;REQUIREMENTS:
;           PACKET LENGTH MUST BE AT LEAST ONE BYTE
;           MINIMUM VALUE THAT CAN BE LOADED INTO BAUD IS 3
;
;THE SOURCE OF DATA TO BE TRANSMITTED IS IN EXTERNAL RAM
;THE STARTING ADDRESS OF THE DATA IS DEFINED BY:
;           MSG_SOURCE (CAN BE UPTO 16 BITS)
;
;THE LENGTH OF OF THE PACKET TO BE TRANSMITTED IS DEFINED BY:
;           MSG_LENGTH (CAN BE UPTO 16 BITS)
;
;DELAY USED WHEN TOO MANY COLLISIONS OCCUR IS DEFINED BY:
;           TWENTIETH_SECOND (1/20 SECOND AT 8MHZ)
;           USER_DELAY (NUMBER OF SECONDS TO DELAY, UPTO 255)
;
;INTERNAL RAM LOCATIONS USED AND MODIFIED:
;           7FH             HIGH_MSG_COUNTER
;           7EH             LOW_MSG_COUNTER
;           7DH             HIGH_MSG_POINTER
;           7CH             LOW_MSG_POINTER
;           0FFH TO 0FAH    NOACK_COUNTER
;           0F9H TO 0F4H    LATE_COUNTER
;           0F3H TO 0EEH    EXCESS_COUNTER
;
;SFRS USED AND MODIFIED:
;           ACC
;           BAUD
;           DPL
;           DPH
;           GMOD
;           IE
;           IEN1
;           IFS
;           MYSLOT
;           RSTAT
;           SLOTTM
;           TCDCNT
;           TCON
;           TH1
;           TFIFO
;           TL1
;           TMOD
;           TSTAT
;
;REGISTERS USED AND MODIFIED:
;           R0
;           R1
;           R5          (USER_SECOND_COUNTER)
;           R6          (ONE_SECOND)
;           R7
;*********************************************************************

$EJ

;*********************************************************************
;THIS PROGRAM CAN BE BROKEN INTO 4 SEPARATE SECTIONS.  THESE ARE: THE
;INTERRUPT VECTORS (LOCATIONS 0 TO 0FFH), GSC INTERRUPT SERVICE
;SOFTWARE (LOCATIONS 100H TO 1FFH), GSC INITIALIZATION SOFTWARE 
;(LOCATIONS 200H TO 2FFH), AND THOSE SECTIONS OF CODE NEEDED TO MAKE 
;THE PROGRAM FUNCTION (LOCATIONS 300H TO 3FFH).  THIS SHOULD ALLOW A 
;USER TO PULL OUT ONLY THE SECTIONS THAT ARE NEEDED TO COMPLETE A 
;SPECIFIC APPLICATION.
;*********************************************************************

;*********************************************************************
;ALL REFERENCES TO ANY TYPE OF ERROR(S) PERTAIN TO THOSE ERRORS THAT 
;ARE DETECTED AND FLAGGED BY THE GSC HARDWARE.  CSMA/CD WAS CHOSEN AND
;IN THIS MODE ALL OF THE TRANSMIT ERRORS, EXCEPT UNDERRUN, CAN OCCUR.
;*********************************************************************
;ALL OF THE CODE REQUIRED TO MAKE THE PROGRAM FLOW PROPERLY IS LOCATED 
;ABOVE LOCATION 300H.  ALL OF THE CODE NEEDED TO SERVICE THE GSC IS 
;LOCATED BETWEEN LOCATIONS 100H TO 200H (EXCEPT FOR THE INTERRUPT VECTORS 
;THEMSELVES).  THE CODE USED TO INITIALIZE THE GSC AND TO BEGIN A 
;TRANSMISSION IS IN LOCATIONS 200H TO 300H.
;*********************************************************************

;*********************************************************************
;THIS PROGRAM WAS WRITTEN TO GO ALONG WITH THE INTEL FAE TRAINING
;MATERIAL FOR THE 8XC152 INTERMEDIATE TRAINING (JAN 1987) BUT SHOULD BE 
;SIMPLE ENOUGH FOR USER WHO HAVE NOT TAKEN THE TRAINING COURSE TO 
;UNDERSTAND. THE LETTERS IN PARENTHESES AND FOLLOWED BY A HYPHEN, (A) -, 
;REFER TO THE LETTERS IN FLOW CHARTS GIVEN DURING THE FAE TRAINING
;*********************************************************************

$EJ
ONE_SECOND              EQU     R6
USER_SECOND_COUNTER     EQU     R5

TWENTIETH_SECOND        EQU     33333   ;1/20 SECOND @ 8MHz
USER_DELAY              EQU     10      ;# OF SECONDS USER WANTS TO
                                        ;WAIT BETWEEN TRANSMISSION
                                        ;WHEN LINK IS CONJESTED
MSG_SOURCE              EQU     40H     ;LOCATION WHERE START OF DATA
                                        ;TO BE TRANSMITTED IS STORED
MSG_LENGTH              EQU     11      ;NUMBER OF BYTES IN EACH PACKET

HIGH_MSG_COUNTER        DATA    7FH
LOW_MSG_COUNTER         DATA    (HIGH_MSG_COUNTER - 1)

HIGH_MSG_POINTER        DATA    (LOW_MSG_COUNTER - 1)
LOW_MSG_POINTER         DATA    (HIGH_MSG_POINTER - 1)

;*********************************************************************
;COUNTERS USED TO TRACK # OF EACH TYPE OF ERROR (6 BYTES EACH) 
;*********************************************************************

NOACK_COUNTER           DATA    (0FFH)
LATE_COUNTER            DATA    (NOACK_COUNTER - 6)
EXCESS_COUNTER          DATA    (LATE_COUNTER - 6)

ORG 0
    JMP INIT1

ORG 43H
XMIT_VALID:
    JMP VALID_XMIT

ORG 4BH
XMIT_ERROR:
    JMP ERROR_XMIT

ORG 1BH
;*********************************************************************
;TIMER1 IS USED TO DELAY TRANSMISSIONS WHEN LINK IS CONJESTED
;*********************************************************************
TIMER1_INT:
    JMP TIMER1_SERVICE

$EJ
ORG 100H
END_OF_MESSAGE?:
;*********************************************************************
; DECIDE IF END OF MESSAGE
;*********************************************************************
    DJNZ LOW_MSG_COUNTER,CONTINUE_XMIT  
    DJNZ HIGH_MSG_COUNTER,CONTINUE_XMIT
                                    ;CHECK FOR END OF MSG

;*********************************************************************
; DISABLE TRANSMIT VALID INTERRUPT
;*********************************************************************
    CLR EGSTV                       ;CLEAR INTERRUPT

;*********************************************************************
; CLEAR PREVIOUS COLLISIONS
;*********************************************************************
    MOV TCDCNT,#0                   ;APPLICABLE TO CSMA/CD ONLY

;*********************************************************************
; REINITIALIZE MESSAGE POINTERS
;*********************************************************************
    MOV LOW_MSG_POINTER,#LOW (MSG_SOURCE)     
    MOV HIGH_MSG_POINTER,#HIGH (MSG_SOURCE)
                                    ;RE-INITIALIZE POINTER
    JMP END_MESSAGE_SERVICE

CONTINUE_XMIT:
;*********************************************************************
; LOAD BYTE
;*********************************************************************
    MOV DPL,LOW_MSG_POINTER
    MOV DPH,HIGH_MSG_POINTER
    MOVX A,@DPTR
    MOV TFIFO,A                     ;MOVE NEXT BYTE INTO TFIFO

;*********************************************************************
; UPDATE POINTERS
;*********************************************************************
    INC DPTR
    MOV LOW_MSG_POINTER,DPL
    MOV HIGH_MSG_POINTER,DPH        ;INCREMENT POINTER AND STORE

;*********************************************************************
; DECIDE IF TFIFO IS FULL (IF NO, JUMP TO VALID_XMIT, IF YES,
; CONTINUE TO VALID_END)
;*********************************************************************
    JB TFNF,END_OF_MESSAGE?         ;CHECK IF TFIFO FULL

END_MESSAGE_SERVICE:
;*********************************************************************
; RETURN TO MAIN PROGRAM LOOP
;*********************************************************************
    RETI

EXAMINE_ERROR:

;*********************************************************************
; (A) - CHECK IF ERROR CAUSED BY NOACK
;*********************************************************************
    JB NOACK,NOACK_SERVICE

;*********************************************************************
; (B) - CHECK IF ERROR CAUSED BY TOO MANY COLLISIONS
;*********************************************************************
    MOV A,TCDCNT
    CJNE A,#0FFH,LATE_COLLISION
    JMP EXCESS_COLLISIONS

NOACK_SERVICE:
;*********************************************************************
; (H) - LOG FAILURE (CAUSED BY MISSING HARDWARE BASED ACKNOWLEDGE)
;       (CSMA/CD ONLY), EGSTV NEEDS TO BE RE-ENABLED TO RETRANSMIT
;       ENTIRE MESSAGE (NECESSARY ONLY FOR NOACK ERROR
;*********************************************************************
    MOV R1,#NOACK_COUNTER
    CALL INCREMENT_COUNTER
    SETB EGSTV
    JMP END_GSC_ERROR

LATE_COLLISION:
;*********************************************************************
; (H) - LOG FAILURE (CAUSED BY COLLISION IN DATA FIELD)
;       (CSMA/CD ONLY)
;*********************************************************************
    MOV R1,#LATE_COUNTER
    CALL INCREMENT_COUNTER
    JMP END_GSC_ERROR

EXCESS_COLLISIONS:
;*********************************************************************
; (C) - LOG FAILURE (CAUSED BY TOO MANY COLLISIONS), (CSMA/CD ONLY)
;*********************************************************************
    MOV R1,#EXCESS_COUNTER
    CALL INCREMENT_COUNTER

TIMER1_START:
;*********************************************************************
; (D) - ENABLE TIMER DELAY
;*********************************************************************
    MOV ONE_SECOND,#20              ;R6 TRACKS NUMBER OF TIMER LOOPS
                                    ;(20) TO DELAY 1 SECOND

    MOV USER_SECOND_COUNTER,#USER_DELAY
                                    ;R5 COUNTS DOWN THE NUMBER OF 
                                    ;SECONDS THE USER WANTS TO DELAY

    MOV TL1,#NOT (LOW TWENTIETH_SECOND)
    MOV TH1,#NOT (HIGH TWENTIETH_SECOND)
                                    ;EACH TIMER OVERFLOW EQUALS 1/20 SEC
                                    ;BASED ON 8 MHZ OSCILLATOR FREQ

    ORL TMOD,#10H                   ;16-BIT TIMER
    SETB ET1                        ;ENABLE TIMER 1 INTERRUPT
    SETB TR1                        ;START TIMER 1

;*********************************************************************
; (E) - DISABLE GSC TRANSMIT INTERRUPTS AND RETURN TO PROGRAM
;*********************************************************************
    CLR EGSTV
    CLR EGSTE
    RETI

END_TIMER_DELAY:
;*********************************************************************
; (F) - ON THE FINAL TIMER INTERRUPT THE PROGRAM JUMPS TO THIS LABEL
;*********************************************************************
    CLR TR1
    CLR ET1

;*********************************************************************
; (G) - ENABLE GSC INTERRUPTS
;*********************************************************************
    SETB EGSTV
    SETB EGSTE
    
END_GSC_ERROR:
;*********************************************************************
; (I) - CLEAR PREVIOUS COLLISIONS (CSMA/CD ONLY)
;*********************************************************************
    MOV TCDCNT, #0                  ;CLEAR COLLISION COUNTER SFR

;*********************************************************************
; (J) - REINITIALIZE POINTERS AND GSC
;*********************************************************************
    MOV HIGH_MSG_POINTER,#HIGH (MSG_SOURCE)
    MOV LOW_MSG_POINTER,#LOW (MSG_SOURCE)
                                    ;START OF TRANSMIT DATA STORAGE

;*********************************************************************
;IN THIS EXAMPLE FIXED PACKET LENGTHS ARE USED SO AN IMMEDIATE VALUE
;CAN BE LOADED INTO THE MESSAGE COUNTER.  IF A VARIABLE LENGTH PACKET
;IS USED THEN THE CALCULATED LENGTH NEEDS TO BE BROKEN INTO TWO BYTES.
;EACH BYTE IS THEN INCREMENTED BY 1.  IF THE BYTE OVERFLOWS FROM 0FFH
;TO 0, NOTHING IS DONE WITH THE CARRY AND 0 SHOULD BE LOADED INTO THE
;APPROPRIATE BYTE.
;*********************************************************************
    MOV HIGH_MSG_COUNTER,#HIGH (MSG_LENGTH) + 1
    MOV LOW_MSG_COUNTER,#LOW (MSG_LENGTH) + 1
                                    ;LENGTH OF TRANSMIT MESSAGE

;*********************************************************************
; (K) - START TRANSMISSION
;*********************************************************************
    SETB EGSTV                      ;EGSTV MUST BE ENABLED TO RETRANSMIT
                                    ;ENTIRE PACKET

    SETB TEN                        ;ENABLE TRANSMITTER/CLEAR ERROR BITS

    JNB TEN,$                       ;WAIT UNTIL TEN IS SET (TEN WILL NOT 
                                    ;BE SET IF A TRANSMISSIONS CRC HAS
                                    ;NOT YET COMPLETED BUT TEN MIGHT BE
                                    ;CLEARED BEFORE CRC COMPLETES)

;*********************************************************************
; (L) - RETURN TO MAIN PROGRAM (ACTUALLY TO "ERROR_XMIT:")
;*********************************************************************
END_ERROR_SERVICE:
    RETI

$EJ
ORG 200H
;*********************************************************************
; USER GENERIC GSC INITIALIZATION
;*********************************************************************
INIT2:
    MOV IFS,#80H                    ;INIT FOR FAST IFS
    MOV BAUD,#60                    ;INIT FOR SLOWEST BAUD RATE
    MOV GMOD,#02H                   ;INIT FOR CSMA/CD, 8-BIT PREAMBLE
                                    ;8-BIT ADDR
    MOV MYSLOT,#80H                 ;INIT FOR DC JAM
    MOV SLOTTM,#0C0H                ;LENGTH OF A SLOT TIME
    MOV TCDCNT,#0                   ;MAKE SURE TCDCNT IS CLEARED

    MOV LOW_MSG_POINTER,#LOW (MSG_SOURCE)
    MOV HIGH_MSG_POINTER,#HIGH (MSG_SOURCE)
                                    ;INIT POINTERS TO START AT THE
                                    ;LOCATION RESERVED FOR DATA TO
                                    ;BE TRANSMITTED.
    MOV LOW_MSG_COUNTER,#0
    MOV HIGH_MSG_COUNTER,#0         ;INIT MESSAGE COUNTERS TO 0

    JB P4.7,GSC_INIT_END            ;ENABLE HBA IF P4.7 = 0
    SETB HABEN

GSC_INIT_END:
    RET                             ;8-BIT PREAMBLE

;*********************************************************************
;DECIDE IF A TRANSMISSION IS WANTED
;*********************************************************************
TRANSMISSION_DECISION:
    CJNE A,#02H,DO_NOT_TRANSMIT     ;TRANSMIT IF ACC=02

;*********************************************************************
; DECIDE IF TRANSMISSION IN PROGRESS
;*********************************************************************
;   MOV A,HIGH_MSG_COUNTER
;   JNZ DO_NOT_TRANSMIT
;   MOV A,LOW_MSG_COUNTER
;   JNZ DO_NOT_TRANSMIT

;*********************************************************************
;IN FAE TRAINING IT WAS POINTED OUT THAT TO DETERMINE IF A TRANSMISSION
;WAS IN PROGRESS THE SOFWARE COULD CHECK THE MESSAGE COUNTER.  THIS WAS
;FOUND NOT TO ALWAYS WORK.  INSTEAD, THE TEN BIT IS CHECKED TO DETERMINE
;IF A TRANSMISSION IS IN PROGRESS.
;*********************************************************************
    JB TEN,DO_NOT_TRANSMIT          ;DO NOT TRANSMIT IF IN PROGRESS 

    JB TEN,DO_NOT_TRANSMIT          ;SECOND TEST IS NEEDED IF TRANSMIT
                                    ;INTERRUPT OCCURS ONE CYCLE PRIOR TO
                                    ;FIRST TEST.  THE INTERRUPT (ERROR)
                                    ;WILL CLEAR TEN FOR FIRST TEST BUT
                                    ;START TRANSMISSION AND WOULD RETURN
                                    ;TO THE NEXT INSTRUCTION WHICH COULD
                                    ;APPEND THE NEW TRANSMISSION. 

;*********************************************************************
; REINITIALIZE COUNTERS JUST PRIOR TO TRANSMISSION
;*********************************************************************
;IN THIS EXAMPLE FIXED PACKET LENGTHS ARE USED SO AN IMMEDIATE VALUE
;CAN BE LOADED INTO THE MESSAGE COUNTER.  IF A VARIABLE LENGTH PACKET
;IS USED THEN THE CALCULATED LENGTH NEEDS TO BE BROKEN INTO TWO BYTES.
;EACH BYTE IS THEN INCREMENTED BY 1.  IF THE BYTE OVERFLOWS FROM 0FFH
;TO 0, NOTHING IS DONE WITH THE CARRY AND 0 SHOULD BE LOADED INTO THE
;APPROPRIATE BYTE.
;*********************************************************************
    MOV HIGH_MSG_COUNTER,#HIGH (MSG_LENGTH) + 1
    MOV LOW_MSG_COUNTER,#LOW (MSG_LENGTH) + 1

;*********************************************************************
; ENABLE TRANSMITTER
;*********************************************************************
    SETB TEN

    JNB TEN,$                       ;WAIT UNTIL TEN IS SET (TEN WILL NOT 
                                    ;BE SET IF A TRANSMISSIONS CRC HAS
                                    ;NOT YET COMPLETED BUT TEN MIGHT BE
                                    ;CLEARED BEFORE CRC COMPLETES)


;*********************************************************************
; ENABLE INTERRUPTS
;*********************************************************************
    SETB EA
    SETB EGSTE                      ;ENABLE ERROR INTERRUPT
    SETB EGSTV                      ;ENABLE TRANSMIT VALID INTERRUPT
    JMP MAIN

DO_NOT_TRANSMIT:
    JMP MAIN

$EJ
ORG 300H
;*********************************************************************
;START OF EXTRA CODE TO MAKE PROGRAM WORK
;*********************************************************************

TIMER1_SERVICE:
;*********************************************************************
;TIMER 1 IS USED TO DELAY TRANSMISSIONS "n" SECONDS WHEN THE LINK IS
;TOO BUSY. "n" IS THE NUMBER OF SECONDS THE USER DETERMINES IS NEEDED.
;TIMER VALUES ARE BASED ON 8 MHZ OSCILLATOR FREQUENCY.
;*********************************************************************
    DJNZ ONE_SECOND,TIMER_LOAD
    DJNZ USER_SECOND_COUNTER,SECOND_LOAD
    JMP END_TIMER_DELAY

SECOND_LOAD:
    MOV ONE_SECOND,#20

TIMER_LOAD:
    MOV TL1,#NOT (LOW TWENTIETH_SECOND)
    MOV TH1,#NOT (HIGH TWENTIETH_SECOND)
    RETI

INCREMENT_COUNTER:
;*********************************************************************
;THIS SECTION OF THE CODE INCREMENTS A COUNTER THAT UTILIZES 6 BYTES
;OF DATA MEMORY TO KEEP TRACK OF THE NUMBER OF COUNTS.  WHEN THIS ROUTINE
;IS CALLED, R1 SHOULD HAVE THE ADDRESS OF THE LOW BYTE OF THE COUNTER
;WHICH IS THE HIGHEST ADDRESS OF THE SIX BYTE STORAGE AREA.  IF THE 
;SIX BYTE COUNTER OVERFLOWS, THE VALUE STORED WILL BE 0FFFFFFFFFFFFH
;AND THE PROGRAM WILL HANG UP UNTIL THE COUNTER IS RESET
;*********************************************************************
    MOV A,@R1                       ;GET LOWEST BYTE OF COUNT
    ADD A,#1                        ;ADD 1
    MOV @R1,A                       ;AND STORE 
    DEC R1  
    MOV A,@R1                       ;GET SECOND BYTE OF COUNT
    ADDC A,#0                       ;ADD 1 IF CARRY FROM PREVIOUS ADD
    MOV @R1,A                       ;AND STORE
    DEC R1                          
    MOV A,@R1                       ;GET THIRD BYTE OF COUNT
    ADDC A,#0                       ;ADD 1 IF CARRY FROM PREVIOUS ADD
    MOV @R1,A                       ;AND STORE
    DEC R1
    MOV A,@R1                       ;GET FOURTH BYTE OF COUNT
    ADDC A,#0                       ;ADD 1 IF CARRY FROM PREVIOUS ADD
    MOV @R1,A                       ;AND STORE
    DEC R1
    MOV A,@R1                       ;GET FIFTH BYTE OF COUNT
    ADDC A,#0                       ;ADD 1 IF CARRY FROM PREVIOUS ADD
    MOV @R1,A                       ;AND STORE
    DEC R1
    MOV A,@R1                       ;GET SIXTH (AND HIGHEST) BYTE OF COUNT
    ADDC A,#0                       ;ADD 1 IF CARRY FROM PREVIOUS ADD
    MOV @R1,A                       ;AND STORE
    JC COUNTER_OVERFLOW             ;CHECK IF CARRY GENERATED ON SIXTH ADD 
    RET

COUNTER_OVERFLOW:
    MOV @R1,#0FFH                   ;IF OVERFLOW RESTORE 0FFH TO BYTE 6
    INC R1
    MOV @R1,#0FFH                   ;STORE 0FFH IN BYTE 5 OF COUNT
    INC R1
    MOV @R1,#0FFH                   ;STORE 0FFH IN BYTE 4 OF COUNT
    INC R1
    MOV @R1,#0FFH                   ;STORE 0FFH IN BYTE 3 OF COUNT
    INC R1
    MOV @R1,#0FFH                   ;STORE 0FFH IN BYTE 2 OF COUNT
    INC R1
    MOV @R1,#0FFH                   ;STORE 0FFH IN BYTE 1 OF COUNT

COUNTER_SERVICE_LOOP:
    MOV A,@R1                       ;GET BYTE 1 OF COUNT
    JNZ COUNTER_SERVICE_LOOP        ;LOOP UNTIL COUNTER SERVICED
                                    ;STILL NEED TO FIGURE OUT HOW THIS
                                    ;WILL BE DONE
    RET

    
VALID_XMIT:
    CALL END_OF_MESSAGE?            ;EGSTV SERVICE ROUTINE

VALID_END:
    RET                             ;RETURN TO MAIN PROGRAM LOOP 


ERROR_XMIT:
    CALL EXAMINE_ERROR

ERROR_END:
    RET



INIT1:
;*********************************************************************
; USER GENERIC INITIALIZATION - USER SHOULD PLACE ALL INITIALIZATION
; SOFTWARE HERE
;*********************************************************************
    MOV R0,#0FFH
    MOV R7,#18                      ;# OF BYTES TO INITIALIZE
INIT_COUNTER_LOOP:
    MOV A,#0
    MOV @R0,A                       ;CLEAR ERROR COUNTER AREA
    DEC R0
    DJNZ R7,INIT_COUNTER_LOOP

    CALL INIT2                      ;GSC INITIALIZATION

MAIN:
;*********************************************************************
; USER MAIN PROGRAM LOOP
;*********************************************************************

;*********************************************************************
; LOAD TRANSMISSION DECISION CRITERIA INTO ACCUMULATOR
;*********************************************************************
    MOV A,PRBS                      ;OBTAIN A RANDOM #
    ANL A,#0FH                      ;MASK OFF UPPER 2 BITS
    JMP TRANSMISSION_DECISION

END
